@@ -1,5 +1,5 @@
 import random, sys
-
+from cStringIO import StringIO
 from persistent import Persistent
 from zope import interface 
 from zope.event import notify
@@ -125,6 +125,60 @@ class Workflow(Persistent, Contained):
         out.append("}")
 
         return "\n".join(out)
+
+    def toDot( self ):
+        """
+        fancier export workflow as dot,
+
+        transition colors
+        
+         - automatic triggers in green
+         - system triggers in yellow if no conditions
+         - system triggers in blue if conditional
+         - manual triggers in red
+
+        state colors
+        
+         - end states in red
+         
+        """
+        
+        io = StringIO()
+        states = set()
+        end_states = set()
+        print >> io, "digraph workflow {"
+
+        for state, transitions in self._sources.items():
+            states.add( state )
+            for tid, t in transitions.items():
+                option = []
+                states.add(  t.destination )
+                if t.destination not in self._sources:
+                    end_states.add( t.destination )
+                if t.trigger is interfaces.AUTOMATIC:
+                    option.append( 'color=green' )
+                elif t.trigger is interfaces.SYSTEM:
+                    if not t.condition in (None, NullCondition):
+                        option.append( 'color=yellow' )
+                    else:
+                        option.append('color=blue')
+                elif not t.condition in ( None, NullCondition):
+                    option.append( 'color=red' )
+
+                print >> io, '  %s -> %s [label="%s", %s];'%(t.source,
+                                                             t.destination,
+                                                             t.transition_id,
+                                                             ', '.join( option ) )
+
+        for state in states:
+            if state in end_states:
+                print >> io, " %s [color=red];"%state
+            else:
+                print >> io, "  %s [shape=box ]; "%state
+        print >> io, " }"
+
+        return io.getvalue()
+
         
 class WorkflowState(object):
     
